﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Runtime.Serialization;
using HeuristicLab.Common;
using HeuristicLab.DataAccess.Interfaces;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataAccess {
  [StorableClass]
  [DataContract]
  [Serializable]
  public class PersistableObject : IDeepCloneable, IPersistableObject {
    [Storable]
    [DataMember]
    public Guid Id { get; set; }

    public PersistableObject() { }
    [StorableConstructor]
    public PersistableObject(bool deserializing) { }
    protected PersistableObject(PersistableObject original, Cloner cloner) {
      cloner.RegisterClonedObject(original, this);
      this.Id = original.Id;
    }
    public virtual IDeepCloneable Clone(Cloner cloner) {
      return new PersistableObject(this, cloner);
    }
    public object Clone() {
      return Clone(new Cloner());
    }

    public override bool Equals(object obj) {
      if (obj is IPersistableObject)
        return this.Id.Equals(
          (obj as IPersistableObject).Id);
      else
        return base.Equals(obj);
    }

    public override string ToString() {
      return Id.ToString();
    }

    public override int GetHashCode() {
      return this.Id.GetHashCode();
    }
  }
}
